@@ -84,16 +84,16 @@ from aio_stdout import IOLock, ainput, aprint
 
 async def countdown(n: int) -> None:
     """Count down from `n`, taking `n` seconds to run."""
-    async with IOLock(n=5) as io_lock:
+    async with IOLock(n=5) as lock:
         for i in range(n, 0, -1):
-            await io_lock.aprint(i)
+            await lock.aprint(i)
             await asyncio.sleep(1)
 
 async def get_name() -> str:
     """Ask the user for their name."""
-    async with IOLock() as io_lock:
-        name = await io_lock.ainput("What is your name? ")
-        await io_lock.aprint(f"Your name is {name}.")
+    async with IOLock() as lock:
+        name = await lock.ainput("What is your name? ")
+        await lock.aprint(f"Your name is {name}.")
     return name
 
 async def main() -> None:
@@ -143,9 +143,40 @@ from `main` before terminating.
 ```python
 from aio_stdout import flush
 
+@flush
 async def main() -> None:
-    async with flush:
-        pass
+    ...
+```
+
+Final Example
+-------------
+
+Combining all best practices, the final example should look something
+like this:
+
+```python
+import asyncio
+from aio_stdout import IOLock, ainput, aprint, flush
+
+async def countdown(n: int) -> None:
+    """Count down from `n`, taking `n` seconds to run."""
+    for i in range(n, 0, -1):
+        await aprint(i)
+        await asyncio.sleep(1)
+
+async def get_name() -> str:
+    """Ask the user for their name."""
+    async with IOLock() as lock:
+        name = await lock.ainput("What is your name? ")
+        await lock.aprint(f"Your name is {name}.")
+    return name
+
+@flush
+async def main() -> None:
+    await asyncio.gather(countdown(15), get_name())
+
+if __name__ == "__main__":
+    asyncio.run(main())
 ```
 
 Common Gotchas
@@ -154,90 +185,81 @@ Common Gotchas
 - Using `input` or `print` instead of `ainput` and `aprint` will push
   a message immediately to the console, potentially conflicting with
   `ainput` or `aprint`.
-- Using `ainput` or `aprint` instead of `io_lock.ainput` and
-  `io_lock.aprint` may produce **deadlock** due to having to
-  wait for the lock to release. As such, the `io_lock` is equipped
+- Using `ainput` or `aprint` instead of `lock.ainput` and
+  `lock.aprint` may produce **deadlock** due to having to
+  wait for the lock to release. As such, the `lock` is equipped
   with a default `timeout` limit of 10 seconds to avoid deadlock
   and explain to users this potential problem.
 '''
-from __future__ import annotations
+import asyncio
+import functools
+import logging
+import sys
+import typing
+
 from asyncio import Queue
 from enum import Enum
 from functools import partial
-from typing import Any, ClassVar, Dict, Generic, IO, Literal, Optional, Tuple, Type, TypedDict, TypeVar
+from types import TracebackType
+from typing import Any, ClassVar, Generic, IO, NoReturn, Optional, TypeVar, Union, get_type_hints, overload
 
-import asyncio
-import logging
-import sys
+if sys.version_info < (3, 8):
+    from builtins import dict as PrintKwargs
+else:
+    class PrintKwargs(typing.TypedDict, total=False):
+        sep: Optional[str]
+        end: Optional[str]
+        file: IO
+        flush: Any
+
+# Make `asyncio.Queue` generic for type-hinting.
+if sys.version_info < (3, 9):
+    from asyncio import Queue as IOQueueType
+    from typing import Dict, Tuple, Type
+else:
+    from builtins import dict as Dict, tuple as Tuple, type as Type
+    IOQueueType = Queue[Tuple[bool, Optional[asyncio.Event], Tuple[str, ...], PrintKwargs]]
+
+if sys.version_info < (3, 11):
+    from typing import NoReturn as Never
+    Self = TypeVar("Self", bound="IOLock")
+else:
+    from typing import Never, Self
 
 __all__ = ["IOLock", "ainput", "aprint", "flush"]
 
 T = TypeVar("T")
+ET = TypeVar("ET", bound=BaseException)
 
-# Make `asyncio.Queue` generic for type-hinting.
-if sys.version_info < (3, 9):
+if sys.version_info < (3, 10):
+    FT = TypeVar("FT", bound=typing.Callable[..., typing.Awaitable[Any]])
 
-    class Queue(Queue, Generic[T]):
-        """
-        A queue, useful for coordinating producer and consumer coroutines.
+    class AsyncContextDecorator:
 
-        If maxsize is less than or equal to zero, the queue size is infinite. If it
-        is an integer greater than 0, then "await put()" will block when the
-        queue reaches maxsize, until an item is removed by get().
+        def __call__(self: "AsyncContextDecorator", func: FT) -> FT:
+            @functools.wraps(func)
+            async def wrapper(*args: Any, **kwargs: Any) -> Any:
+                async with self:
+                    return await func(*args, **kwargs)
+            return wrapper
 
-        Unlike the standard library Queue, you can reliably know this Queue's size
-        with qsize(), since your single-threaded asyncio application won't be
-        interrupted between calling qsize() and doing an operation on the Queue.
-        """
-        __slots__ = ()
+        def __aenter__(self: "AsyncContextDecorator") -> Any:
+            raise NotImplementedError
 
-        async def get(self: Queue[T], /) -> T:
-            """
-            Remove and return an item from the queue.
-
-            If queue is empty, wait until an item is available.
-            """
-            return await super().get()
-
-        def get_nowait(self: Queue[T], /) -> T:
-            """
-            Remove and return an item from the queue.
-
-            Return an item if one is immediately available, else raise QueueEmpty.
-            """
-            return super().get_nowait()
-
-        async def put(self: Queue[T], item: T, /) -> T:
-            """
-            Put an item into the queue.
-
-            Put an item into the queue. If the queue is full, wait until a free
-            slot is available before adding item.
-            """
-            return await super().put(item)
-
-        def put_nowait(self: Queue[T], item: T, /) -> T:
-            """
-            Put an item into the queue without blocking.
-
-            If no free slot is immediately available, raise QueueFull.
-            """
-            return super().put_nowait(item)
+        def __aexit__(
+            self: "AsyncContextDecorator",
+            exc_type: Optional[Type[ET]],
+            exc_value: Optional[ET],
+            traceback: Optional[TracebackType],
+        ) -> Any:
+            raise NotImplementedError
 
+else:
+    from contextlib import AsyncContextDecorator
 
 logger = logging.getLogger(__name__)
 
 
-class PrintKwargs(TypedDict, total=False):
-    sep: Optional[str]
-    end: Optional[str]
-    file: IO
-    flush: Any
-
-
-IOQueueType = Queue[Tuple[bool, Optional[asyncio.Event], Tuple[str, ...], PrintKwargs]]
-
-
 class IOLock(asyncio.Lock):
     """
     The `IOLock` may be used to control the order with which `ainput` and
@@ -252,7 +274,7 @@ class IOLock(asyncio.Lock):
     Attributes
     -----------
     Construct an IOLock using:
-        >>> io_lock = IOLock(n=..., timeout=...)
+        >>> lock = IOLock(n=..., timeout=...)
     By default, `n = None` and `timeout = 10`.
 
     n:
@@ -268,26 +290,31 @@ class IOLock(asyncio.Lock):
     --------
     Use it as a context manager to ensure you can't have printed messages
     in-between them.
-        >>> async with IOLock() as io_lock:
-        ...     name = await io_lock.ainput("What is your name? ")
-        ...     await io_lock.aprint(f"Your name is {name}.")
+        >>> async with IOLock() as lock:
+        ...     name = await lock.ainput("What is your name? ")
+        ...     await lock.aprint(f"Your name is {name}.")
         ...
         What is your name? (...)
         Your name is (...).
 
     WARNING
     --------
-    Using `aprint` with `block=True` or `ainput` inside of an `io_lock`
+    Using `aprint` with `block=True` or `ainput` inside of an `lock`
     block will cause deadlock, preventing your program from continuing.
-    Use `io_lock.ainput` and `io_lock.aprint` instead.
+    Use `lock.ainput` and `lock.aprint` instead.
 
-    Using `aprint` with `block=False` inside of an `io_lock` block
-    will delay the `aprint` until the `io_lock` block is finished.
+    Using `aprint` with `block=False` inside of an `lock` block will
+    delay the `aprint` until the `lock` block is finished.
 
-    With the default `io_lock.timeout` however, such deadlocks only hold for 10 seconds.
+    With the default `lock.timeout` however, such deadlocks only hold for 10 seconds.
     """
     _class_is_finished: ClassVar[asyncio.Event] = asyncio.Event()
-    _class_queue: ClassVar[Queue[Tuple[Optional[float], IOQueueType, asyncio.Event, asyncio.Event]]] = Queue()
+
+    if sys.version_info < (3, 9):
+        _class_queue: ClassVar[Queue] = Queue()
+    else:
+        _class_queue: ClassVar[Queue[Tuple[Optional[float], IOQueueType, asyncio.Event, asyncio.Event]]] = Queue()
+
     _i: int
     _is_awake: asyncio.Event
     _is_finished: asyncio.Event
@@ -300,7 +327,7 @@ class IOLock(asyncio.Lock):
     # Finished running IO because there's nothing being ran yet.
     _class_is_finished.set()
 
-    def __init__(self: IOLock, /, *args: Any, n: Optional[int] = None, timeout: Optional[float] = 10, **kwargs: Any) -> None:
+    def __init__(self: Self, *args: Any, n: Optional[int] = None, timeout: Optional[float] = 10, **kwargs: Any) -> None:
         if n is not None and not isinstance(n, int):
             raise TypeError(f"n must be an integer or None, got {x!r}")
         elif timeout is not None and not isinstance(timeout, (int, float)):
@@ -321,12 +348,26 @@ class IOLock(asyncio.Lock):
         # Finished running IO because there's nothing being ran yet.
         self._is_finished.set()
 
-    async def __aenter__(self: IOLock, /) -> IOLock:
+    async def __aenter__(self: Self) -> Self:
         """Acquire the lock and return itself."""
         await super().__aenter__()
         return self
 
-    async def acquire(self: IOLock, /) -> Literal[True]:
+    @overload
+    async def acquire(self: Self, _: Never) -> NoReturn:
+        ...
+
+    if sys.version_info < (3, 8):
+        @overload
+        async def acquire(self: Self) -> bool:
+            ...
+
+    else:
+        @overload
+        async def acquire(self: Self) -> typing.Literal[True]:
+            ...
+
+    async def acquire(self):
         """
         Acquire a lock.
 
@@ -346,7 +387,7 @@ class IOLock(asyncio.Lock):
             # The lock is sleeping because there's nothing being ran yet.
             self._is_awake.clear()
 
-    def release(self: IOLock, /) -> None:
+    def release(self: Self) -> None:
         """
         Release a lock.
 
@@ -371,7 +412,7 @@ class IOLock(asyncio.Lock):
             self._queue = Queue()
 
     @classmethod
-    async def __exhaust_queue(cls: Type[IOLock], io_queue: IOQueueType, /) -> None:
+    async def __exhaust_queue(cls: Type[Self], io_queue: IOQueueType) -> None:
         """Helper method to exhaust a queue."""
         # Otherwise the io lock is not sleeping and the io queue should be exhausted.
         while not io_queue.empty():
@@ -397,13 +438,13 @@ class IOLock(asyncio.Lock):
             io_queue.task_done()
 
     @classmethod
-    async def __wait_event(cls: Type[IOLock], event: asyncio.Event, message: str, /) -> str:
+    async def __wait_event(cls: Type[Self], event: asyncio.Event, message: str) -> str:
         """Helper method to wait until an event occurs."""
         await event.wait()
         return message
 
     @classmethod
-    async def _execute_io(cls: Type[IOLock], /) -> None:
+    async def _execute_io(cls: Type[Self]) -> None:
         """Helper method for executing IO requests."""
         while not cls._class_queue.empty():
             timeout, io_queue, is_awake, is_finished = await cls._class_queue.get()
@@ -415,27 +456,27 @@ class IOLock(asyncio.Lock):
                 await cls.__exhaust_queue(io_queue)
                 # Sleep once all tasks are done.
                 is_awake.clear()
-                tasks = [is_finished_task]
-                tasks.append(asyncio.create_task(cls.__wait_event(is_awake, "awake")))
+                # Wait until the queue is either finished or awake.
+                tasks = [
+                    is_finished_task,
+                    asyncio.create_task(cls.__wait_event(is_awake, "awake")),
+                ]
                 if timeout is None:
                     as_completed = asyncio.as_completed(tasks)
                 else:
                     as_completed = asyncio.as_completed(tasks, timeout=timeout)
-                # Wait until one of the tasks is done.
                 for task in as_completed:
                     try:
                         task_type = await task
                     except asyncio.TimeoutError:
                         task_type = "timeout"
                     break
-                del tasks[0]
-                for task in tasks:
-                    task.cancel()
-                for task in tasks:
-                    try:
-                        await task
-                    except asyncio.CancelledError:
-                        pass
+                # Stop checking if it is awake or not.
+                tasks[1].cancel()
+                try:
+                    await tasks[1]
+                except asyncio.CancelledError:
+                    pass
             cls._class_queue.task_done()
             # Wake up if finished.
             if task_type == "finished":
@@ -447,10 +488,10 @@ class IOLock(asyncio.Lock):
                 # Warn the user if they timed out after 10 seconds and other IO is waiting.
                 if None is not timeout >= 10 and not (cls._class_queue.empty() and IO_QUEUE.empty()):
                     print(
-                        "An `io_lock` timed out after 10 seconds or more.",
+                        "An `IOLock` timed out after 10 seconds or more.",
                         "This is likely due to the use of `aprint` or `ainput`",
-                        "instead of `io_lock.aprint` or `io_lock.ainput` while",
-                        "inside of an `io_lock` block."
+                        "instead of `lock.aprint` or `lock.ainput` while",
+                        "inside of a `async with lock` block."
                     )
                 # Insert the global queue into the class queue.
                 global_queue = Queue()
@@ -467,7 +508,7 @@ class IOLock(asyncio.Lock):
             IS_FINISHED.clear()
             asyncio.create_task(_execute_io())
 
-    def _schedule_io(self: IOLock, is_print: bool, event: Optional[asyncio.Event], args: Tuple[str, ...], kwargs: Optional[PrintKwargs], /) -> None:
+    def _schedule_io(self: Self, is_print: bool, event: Optional[asyncio.Event], args: Tuple[str, ...], kwargs: Optional[PrintKwargs]) -> None:
         """Helper method for scheduling IO requests."""
         # Insert the next IO request.
         self._queue.put_nowait((is_print, event, args, kwargs))
@@ -491,7 +532,7 @@ class IOLock(asyncio.Lock):
         else:
             self._is_awake.set()
 
-    async def ainput(self: IOLock, /, *args: Any) -> str:
+    async def ainput(self: Self, *args: Any) -> str:
         """Locked version of `ainput`. See `ainput` for more details."""
         # Perform early type-checking on args.
         if len(args) > 1:
@@ -512,7 +553,7 @@ class IOLock(asyncio.Lock):
         else:
             return response
 
-    async def aprint(self: IOLock, /, *args: Any, block: bool = False, **kwargs: Any) -> None:
+    async def aprint(self: Self, *args: Any, block: bool = False, **kwargs: Any) -> None:
         """Locked version of `aprint`. See `aprint` for more details."""
         # Perform early type-checking on kwargs.
         for kwarg, value in kwargs.items():
@@ -537,7 +578,7 @@ class IOLock(asyncio.Lock):
             await asyncio.sleep(0)
 
     @property
-    def n(self: IOLock, /) -> Optional[int]:
+    def n(self: Self) -> Optional[int]:
         """
         The number of io requests that can be queued at a time
         before letting other io requests go through.
@@ -547,7 +588,7 @@ class IOLock(asyncio.Lock):
         return self._n
 
     @property
-    def timeout(self: IOLock, /) -> Optional[float]:
+    def timeout(self: Self) -> Optional[float]:
         """
         The number of seconds the io lock can sleep before letting other
         io requests go through.
@@ -557,14 +598,31 @@ class IOLock(asyncio.Lock):
         return self._timeout
 
 
-class Flush(Enum):
-    """Use `async with flush: ...` to flush all io before exiting."""
+class Flush(AsyncContextDecorator, Enum):
+    """
+    Helper class to block until all io is flushed.
+
+    Decorator Usage:
+        @flush
+        async def main() -> None:
+            ...
+
+    Context Manager Usage:
+        async def main() -> None:
+            async with flush:
+                ...
+    """
     flush = ()
 
-    async def __aenter__(self: Flush, /) -> None:
+    async def __aenter__(self: "Flush") -> None:
         pass
 
-    async def __aexit__(self: Flush, /, *args: Any) -> None:
+    async def __aexit__(
+        self: "Flush",
+        exc_type: Optional[Type[ET]],
+        exc_value: Optional[ET],
+        traceback: Optional[TracebackType],
+    ) -> None:
         """Waits until all IO is flushed."""
         await IOLock._class_is_finished.wait()
         await IS_FINISHED.wait()
@@ -572,7 +630,10 @@ class Flush(Enum):
 
 flush: Flush = Flush.flush
 
-INPUT_RESULTS: Dict[asyncio.Event, Union[Tuple[Literal[False], str], Tuple[Literal[True], Exception]]] = {}
+if sys.version_info < (3, 8):
+    INPUT_RESULTS: Dict[asyncio.Event, Tuple[bool, Union[Exception, str]]] = {}
+else:
+    INPUT_RESULTS: Dict[asyncio.Event, Union[Tuple[typing.Literal[False], str], Tuple[typing.Literal[True], Exception]]] = {}
 IO_QUEUE: IOQueueType = Queue()
 IS_FINISHED: asyncio.Event = asyncio.Event()
 PRINT_EXCEPTIONS: Dict[asyncio.Event, Exception] = {}
@@ -608,7 +669,7 @@ async def _execute_io() -> None:
     # Signal no io requests are being executed.
     IS_FINISHED.set()
 
-def _schedule_io(is_print: bool, event: Optional[asyncio.Event], args: Tuple[str, ...], kwargs: Optional[PrintKwargs], /) -> None:
+def _schedule_io(is_print: bool, event: Optional[asyncio.Event], args: Tuple[str, ...], kwargs: Optional[PrintKwargs]) -> None:
     """Helper function for scheduling IO requests."""
     # Insert the next IO request.
     IO_QUEUE.put_nowait((is_print, event, args, kwargs))
@@ -624,21 +685,18 @@ async def ainput(*args: Any) -> str:
     Blocks the current coroutine from progressing until `input` is given.
 
     WARNING:
-        Using `ainput` inside of an `io_lock` block will cause deadlock,
+        Using `ainput` inside of an `IOLock` block will cause deadlock,
         preventing your program from continuing.
-        Use `io_lock.ainput` instead.
+        Use `lock.ainput` instead.
 
-        With the default `io_lock.timeout` however, such deadlocks only
+        With the default `lock.timeout` however, such deadlocks only
         hold for 10 seconds.
 
     NOTE:
         Since `ainput` only queues a prompt to be printed evantually,
         it may not print anything if the `asyncio` loop terminates first.
         In order to flush out all remaining `aprint`s and `ainput`s, use
-            >>> async with flush:
-            ...     pass  # Main code.
-            ...
-        at the end of the main code to wait until all other code gets to print.
+        `aio_stdout.flush`. See `help(aio_stdout.flush)` for more details.
     """
     # Perform early type-checking on args.
     if len(args) > 1:
@@ -670,24 +728,21 @@ async def aprint(*args: Any, block: bool = False, **kwargs: Any) -> None:
     continuing, such as when printing to a file.
 
     WARNING:
-        Using `aprint` with `block=True` inside of an `io_lock` block
+        Using `aprint` with `block=True` inside of an `IOLock` block
         will cause deadlock, preventing your program from continuing.
-        Use `io_lock.aprint` instead.
+        Use `lock.aprint` instead.
 
-        Using `aprint` with `block=False` inside of an `io_lock` block
-        will delay the `aprint` until the `io_lock` block is finished.
+        Using `aprint` with `block=False` inside of an `IOLock` block
+        will delay the `aprint` until the `lock` block is finished.
 
-        With the default `io_lock.timeout` however, such deadlocks only
+        With the default `lock.timeout` however, such deadlocks only
         hold for 10 seconds.
 
     NOTE:
         Since `aprint` only queues a message to be printed evantually,
         it may not print anything if the `asyncio` loop terminates first.
         In order to flush out all remaining `aprint`s and `ainput`s, use
-            >>> async with flush:
-            ...     pass  # Main code.
-            ...
-        at the end of the main code to wait until all other code gets to print.
+        `aio_stdout.flush`. See `help(aio_stdout.flush)` for more details.
     """
     # Perform early type-checking on kwargs.
     for kwarg, value in kwargs.items():
@@ -709,3 +764,31 @@ async def aprint(*args: Any, block: bool = False, **kwargs: Any) -> None:
     # Wait at least once before returning so that the print can start running.
     else:
         await asyncio.sleep(0)
+
+# Finalize type-hints.
+IOLock.acquire.__annotations__ = {
+    "self": Self,
+    "return":
+        bool if sys.version_info < (3, 8)
+        else typing.Literal[True],
+}
+
+for hinted in (
+    IOLock,
+    IOLock.__init__,
+    IOLock.__aenter__,
+    IOLock.release,
+    IOLock._IOLock__exhaust_queue.__func__,
+    IOLock._IOLock__wait_event.__func__,
+    IOLock._execute_io.__func__,
+    IOLock._schedule_io,
+    IOLock.ainput,
+    IOLock.aprint,
+    IOLock.n.fget,
+    IOLock.timeout.fget,
+    Flush.__aenter__,
+    Flush.__aexit__,
+    ainput,
+    aprint,
+):
+    hinted.__annotations__ = get_type_hints(hinted)
